"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_tag_1 = __importDefault(require("graphql-tag"));
const chai_1 = require("chai");
const getEntityAttributes_1 = require("./getEntityAttributes");
const test_commons_1 = require("../../../../../test/test-commons");
describe('getEntityAttributes', async () => {
    let schema;
    before(async () => {
        schema = await test_commons_1.getMockSchema();
    });
    it('derives a list of entity attributes', () => {
        const attributes = getEntityAttributes_1.getEntityAttributes(QUERY_NODE_PET_LIST, schema, 'id');
        chai_1.expect(attributes
            .map(({ gqlType: _gqlType, ...attr }) => ({ ...attr })).sort())
            .to.deep.equal(EXPECTED_ATTRS_PET_LIST.sort());
    });
    it('relation field is marked as such', () => {
        const attributes = getEntityAttributes_1.getEntityAttributes(QUERY_NODE_PET_LIST, schema, 'id');
        const ownerAttr = attributes.find(attr => attr.name === 'owner');
        chai_1.expect(ownerAttr?.isSingleRelationField).to.eq(true);
    });
    it('relation field has entity name', () => {
        const attributes = getEntityAttributes_1.getEntityAttributes(QUERY_NODE_PET_LIST, schema, 'id');
        const ownerAttr = attributes.find(attr => attr.name === 'owner');
        chai_1.expect(ownerAttr?.type).to.eq('OwnerDTO');
    });
    it('should define correct relation type for mandatory relation field', function () {
        const attributes = getEntityAttributes_1.getEntityAttributes(QUERY_NODE_VISIT_LIST, schema, 'id');
        const petAttr = attributes.find(attr => attr.name === 'pet');
        chai_1.expect(petAttr.isSingleRelationField).to.be.true;
        chai_1.expect(attributes
            .map(({ gqlType: _gqlType, ...attr }) => ({ ...attr })).sort())
            .to.deep.equal([
            {
                kind: 'gql',
                name: 'description',
                displayName: 'Description',
                isSingleRelationField: false,
                isMultiRelationField: false,
                type: 'String',
                isDateType: false,
                isEnumType: false,
                isNumberType: false,
                enumOptions: undefined,
                nestedAttributes: undefined,
            },
            {
                kind: 'gql',
                name: 'pet',
                displayName: 'Pet',
                isSingleRelationField: true,
                isMultiRelationField: false,
                type: 'PetDTO',
                isDateType: false,
                isEnumType: false,
                isNumberType: false,
                enumOptions: undefined,
                nestedAttributes: [
                    "birthDate",
                    "description",
                    "diseases",
                    "id",
                    "identificationNumber",
                    "owner",
                    "tags",
                    "type",
                    "weightInGrams"
                ],
            },
            {
                kind: 'gql',
                name: 'visitEnd',
                displayName: 'Visit End',
                isSingleRelationField: false,
                isMultiRelationField: false,
                type: 'LocalDateTime',
                isDateType: true,
                isEnumType: false,
                isNumberType: false,
                enumOptions: undefined,
                nestedAttributes: undefined
            },
            {
                kind: 'gql',
                name: 'visitStart',
                displayName: 'Visit Start',
                isSingleRelationField: false,
                isMultiRelationField: false,
                type: 'LocalDateTime',
                isDateType: true,
                isEnumType: false,
                isNumberType: false,
                enumOptions: undefined,
                nestedAttributes: undefined
            }
        ]);
    });
    it('query contains id attribute with custom name', () => {
        const attributes = getEntityAttributes_1.getEntityAttributes(QUERY_WITHOUT_ID, schema, 'identificationNumber');
        const dateAttr = attributes.find(attr => attr.name === 'birthDate');
        chai_1.expect(dateAttr?.type).to.eq('Date');
    });
    it('fails if there is no "id" attribute in query', () => {
        chai_1.expect(() => getEntityAttributes_1.getEntityAttributes(QUERY_WITHOUT_ID, schema, 'id'))
            .to.throw(test_commons_1.stripNewLines(`
Attributes of query 'pet' did not contain id attribute with name 'id', which required for screen generation. 
If you are using entity with id field with name distinct from 'id', 
please be sure that you pass information about id field name in answers ('idField', 'listIdField', 'detailsIdField')`));
    });
    it("should put enum options in entity attributes", function () {
        const attributes = getEntityAttributes_1.getEntityAttributes(QUERY_WITH_ENUM, schema, 'id');
        const defenseStatusAttr = attributes.find(attr => attr.name === 'defenseStatus');
        chai_1.expect(defenseStatusAttr.isEnumType).to.be.true;
        chai_1.expect(defenseStatusAttr.enumOptions.map(en => en.name).sort())
            .to.eql(['NEEDS_PROTECTION', 'NO_DANGER', 'RED_BOOK']);
    });
});
const QUERY_WITHOUT_ID = graphql_tag_1.default(`
  query Get_Pet($id: BigInteger) {
    pet(id: $id) {
      identificationNumber
      birthDate
    }
  }
`);
const QUERY_WITH_ENUM = graphql_tag_1.default(`
  query Get_Pet_Type($id: ID!) {
    petType(id: $id) {
      id
      defenseStatus
      name
    }
  }
`);
const QUERY_NODE_VISIT_LIST = graphql_tag_1.default(`
    query Get_Visit_List {
      visitList {
        description
        id
        pet {
          id
          identificationNumber
          birthDate
        }
        visitEnd
        visitStart
      }
    }`);
const QUERY_NODE_PET_LIST = graphql_tag_1.default(`
    query Get_Pet_List {
      petList {
        id
        identificationNumber
        birthDate
        type {
          id
          name
        }
        owner {
          id
          firstName
          lastName
        }
        description {
          id
          description
        }
        tags {
          id
          name
        }
        diseases {
          id
          name
          description
        }  
      }
    }
`);
const EXPECTED_ATTRS_PET_LIST = [
    {
        "displayName": "Identification Number",
        "enumOptions": undefined,
        "isMultiRelationField": false,
        "isSingleRelationField": false,
        "kind": "gql",
        "name": "identificationNumber",
        "nestedAttributes": undefined,
        "type": "String",
        "isDateType": false,
        "isEnumType": false,
        "isNumberType": false,
    },
    {
        "displayName": "Birth Date",
        "enumOptions": undefined,
        "isMultiRelationField": false,
        "isSingleRelationField": false,
        "kind": "gql",
        "name": "birthDate",
        "nestedAttributes": undefined,
        "type": "Date",
        "isDateType": true,
        "isEnumType": false,
        "isNumberType": false,
    },
    {
        "displayName": "Type",
        "enumOptions": undefined,
        "isMultiRelationField": false,
        "isSingleRelationField": true,
        "kind": "gql",
        "name": "type",
        "nestedAttributes": [
            "defenseStatus",
            "id",
            "name",
        ],
        "type": "PetTypeDTO",
        "isDateType": false,
        "isEnumType": false,
        "isNumberType": false,
    },
    {
        "displayName": "Owner",
        "enumOptions": undefined,
        "isMultiRelationField": false,
        "isSingleRelationField": true,
        "kind": "gql",
        "name": "owner",
        "nestedAttributes": [
            "address",
            "city",
            "email",
            "firstName",
            "id",
            "lastName",
            "telephone",
        ],
        "type": "OwnerDTO",
        "isDateType": false,
        "isEnumType": false,
        "isNumberType": false,
    },
    {
        "displayName": "Description",
        "enumOptions": undefined,
        "isMultiRelationField": false,
        "isSingleRelationField": true,
        "kind": "gql",
        "name": "description",
        "nestedAttributes": [
            "description",
            "id",
        ],
        "type": "PetDescriptionDTO",
        "isDateType": false,
        "isEnumType": false,
        "isNumberType": false,
    },
    {
        "displayName": "Tags",
        "enumOptions": undefined,
        "isMultiRelationField": true,
        "isSingleRelationField": false,
        "kind": "gql",
        "name": "tags",
        "nestedAttributes": [
            "id",
            "name",
        ],
        "type": "TagDTO",
        "isDateType": false,
        "isEnumType": false,
        "isNumberType": false,
    },
    {
        "displayName": "Diseases",
        "enumOptions": undefined,
        "isMultiRelationField": true,
        "isSingleRelationField": false,
        "kind": "gql",
        "name": "diseases",
        "nestedAttributes": [
            "description",
            "name",
            "id",
        ],
        "type": "PetDiseaseDTO",
        "isDateType": false,
        "isEnumType": false,
        "isNumberType": false,
    }
];
//# sourceMappingURL=getEntityAttributes.test.js.map